@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998-2001, 2007, 2009-2011 Free Software Foundation, Inc.
+    Copyright (C) 1998-2001, 2007, 2009-2014 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -84,8 +84,8 @@ struct __attribute__ ((packed)) _FatDirEntry {
 };
 
 struct _FatSpecific {
-	FatBootSector	boot_sector;    /* structure of boot sector */
-	FatInfoSector	info_sector;    /* fat32-only information sector */
+	FatBootSector	*boot_sector;    /* structure of boot sector */
+	FatInfoSector	*info_sector;    /* fat32-only information sector */
 
 	int		logical_sector_size;	/* illogical sector size :-) */
 	PedSector	sector_count;
@@ -156,10 +156,7 @@ extern void fat_print (const PedFileSystem* fs);
 extern PedFileSystem* fat_alloc (const PedGeometry* geom);
 extern void fat_free (PedFileSystem* fs);
 extern int fat_alloc_buffers (PedFileSystem* fs);
-extern void fat_free_buffers (PedFileSystem* fs);
 
 extern int fat_resize (PedFileSystem* fs, PedGeometry* geom, PedTimer* timer);
 
-extern int fat_set_frag_sectors (PedFileSystem* fs, PedSector frag_sectors);
-
 #endif /* FAT_H_INCLUDED */
