@@ -1,7 +1,7 @@
 /* -*- Mode: c; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2000-2001, 2007-2011 Free Software Foundation, Inc.
+    Copyright (C) 2000-2001, 2007-2014 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -167,7 +167,7 @@ aix_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
         return 0;
 }
 
-static int
+static int _GL_ATTRIBUTE_CONST
 aix_partition_get_flag (const PedPartition* part, PedPartitionFlag flag)
 {
         return 0;
@@ -195,7 +195,7 @@ aix_get_max_supported_partition_count (const PedDisk* disk, int *max_n)
 	return true;
 }
 
-static int
+static int _GL_ATTRIBUTE_PURE
 aix_partition_align (PedPartition* part, const PedConstraint* constraint)
 {
         PED_ASSERT (part != NULL);
@@ -203,13 +203,13 @@ aix_partition_align (PedPartition* part, const PedConstraint* constraint)
         return 1;
 }
 
-static int
+static int _GL_ATTRIBUTE_PURE
 aix_partition_enumerate (PedPartition* part)
 {
 	return 1;
 }
 
-static int
+static int _GL_ATTRIBUTE_PURE
 aix_alloc_metadata (PedDisk* disk)
 {
 	return 1;
