@@ -1,7 +1,7 @@
 /* -*- Mode: c; indent-tabs-mode: nil -*-
 
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2007, 2009-2012 Free Software Foundation, Inc.
+    Copyright (C) 2007, 2009-2014 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,27 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
+#include <inttypes.h>
+#include <uuid/uuid.h>
+
+/* hack: use the ext2 uuid library to generate a reasonably random (hopefully
+ * with /dev/random) number.  Unfortunately, we can only use 4 bytes of it.
+ * We make sure to avoid returning zero which may be interpreted as no FAT
+ * serial number or no MBR signature.
+ */
+static inline uint32_t
+generate_random_uint32 (void)
+{
+       union {
+               uuid_t uuid;
+               uint32_t i;
+       } uu32;
+
+       uuid_generate (uu32.uuid);
+
+       return uu32.i > 0 ? uu32.i : 0xffffffff;
+}
+
 /* Return nonzero if FS_TYPE_NAME starts with "linux-swap".
    This must match the NUL-terminated "linux-swap" as well
    as "linux-swap(v0)" and "linux-swap(v1)".  */
