@@ -1,7 +1,7 @@
 /* -*- Mode: c; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2000-2001, 2005, 2007-2009 Free Software Foundation, Inc.
+    Copyright (C) 2000-2001, 2005, 2007-2014 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -43,6 +43,9 @@
 #define SUN_DISK_MAGIC		0xDABE	/* Disk magic number */
 #define SUN_DISK_MAXPARTITIONS	8
 
+#define SUN_VTOC_VERSION	1
+#define SUN_VTOC_SANITY		0x600DDEEE
+
 #define WHOLE_DISK_ID		0x05
 #define WHOLE_DISK_PART		2	/* as in 0, 1, 2 (3rd partition) */
 #define LINUX_SWAP_ID		0x82
@@ -67,9 +70,18 @@ struct __attribute__ ((packed)) _SunPartitionInfo {
 
 struct __attribute__ ((packed)) _SunRawLabel {
 	char 		info[128];	/* Informative text string */
-	u_int8_t	spare0[14];
+	u_int32_t	version;	/* Layout version */
+	u_int8_t	volume[8];	/* Volume name */
+	u_int16_t	nparts;		/* Number of partitions */
 	SunPartitionInfo infos[SUN_DISK_MAXPARTITIONS];
-	u_int8_t	spare1[246];	/* Boot information etc. */
+	u_int16_t	padding;	/* Alignment padding */
+	u_int32_t	bootinfo[3];	/* Info needed by mboot */
+	u_int32_t	sanity;		/* To verify vtoc sanity */
+	u_int32_t	reserved[10];	/* Free space */
+	u_int32_t	timestamp[8];	/* Partition timestamp */
+	u_int32_t	write_reinstruct; /* sectors to skip, writes */
+	u_int32_t	read_reinstruct; /* sectors to skip, reads */
+	u_int8_t	spare1[148];	/* Padding */
 	u_int16_t	rspeed;		/* Disk rotational speed */
 	u_int16_t	pcylcount;	/* Physical cylinder count */
 	u_int16_t	sparecyl;	/* extra sects per cylinder */
@@ -96,7 +108,6 @@ struct _SunPartitionData {
 struct _SunDiskData {
 	PedSector		length; /* This is based on cyl - alt-cyl */
 	SunRawLabel		raw_label;
-	int                     cylinder_alignment;
 };
 
 static PedDiskType sun_disk_type;
@@ -129,7 +140,7 @@ sun_verify_checksum (SunRawLabel const *label)
 static int
 sun_probe (const PedDevice *dev)
 {
-	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (dev != NULL);
 
 	void *s0;
 	if (!ptt_read_sector (dev, 0, &s0))
@@ -156,25 +167,6 @@ sun_probe (const PedDevice *dev)
 	return ok;
 }
 
-#ifndef DISCOVER_ONLY
-static int
-sun_clobber (PedDevice* dev)
-{
-	PED_ASSERT (dev != NULL, return 0);
-	PED_ASSERT (sun_probe (dev), return 0);
-
-	void *s0;
-	if (!ptt_read_sector (dev, 0, &s0))
-		return 0;
-
-	SunRawLabel *table = s0;
-	table->magic = 0;
-	int write_ok = ped_device_write (dev, (void*) table, 0, 1);
-	free (s0);
-	return write_ok;
-}
-#endif /* !DISCOVER_ONLY */
-
 static PedDisk*
 sun_alloc (const PedDevice* dev)
 {
@@ -183,7 +175,7 @@ sun_alloc (const PedDevice* dev)
 	SunDiskData*	sun_specific;
 	const PedCHSGeometry*	bios_geom = &dev->bios_geom;
 	PedSector	cyl_size = bios_geom->sectors * bios_geom->heads;
-	PED_ASSERT (cyl_size != 0, return NULL);
+	PED_ASSERT (cyl_size != 0);
 
         disk = _ped_disk_alloc (dev, &sun_disk_type);
 	if (!disk)
@@ -194,10 +186,8 @@ sun_alloc (const PedDevice* dev)
 		goto error_free_disk;
 	sun_specific = (SunDiskData*) disk->disk_specific;
 
-	PED_ASSERT (bios_geom->cylinders == (PedSector) (dev->length / cyl_size),
-                    return NULL);
+	PED_ASSERT (bios_geom->cylinders == (PedSector) (dev->length / cyl_size));
 	sun_specific->length = ped_round_down_to (dev->length, cyl_size);
-        sun_specific->cylinder_alignment = 1;
 
 	label = &sun_specific->raw_label;
 	memset(label, 0, sizeof(SunRawLabel));
@@ -213,6 +203,10 @@ sun_alloc (const PedDevice* dev)
 	label->nsect	= PED_CPU_TO_BE16 (bios_geom->sectors);
 	label->ncyl	= PED_CPU_TO_BE16 (dev->length / cyl_size);
 
+	label->sanity   = PED_CPU_TO_BE32 (SUN_VTOC_SANITY);
+	label->version  = PED_CPU_TO_BE32 (SUN_VTOC_VERSION);
+	label->nparts   = PED_CPU_TO_BE16 (SUN_DISK_MAXPARTITIONS);
+
 	/* Add a whole disk partition at a minimum */
 	label->infos[WHOLE_DISK_PART].id = WHOLE_DISK_ID;
 	label->partitions[WHOLE_DISK_PART].start_cylinder = 0;
@@ -259,42 +253,6 @@ sun_free (PedDisk *disk)
 	_ped_disk_free (disk);
 }
 
-static int
-sun_disk_set_flag (PedDisk *disk, PedDiskFlag flag, int state)
-{
-        SunDiskData *disk_specific = disk->disk_specific;
-        switch (flag) {
-        case PED_DISK_CYLINDER_ALIGNMENT:
-                disk_specific->cylinder_alignment = !!state;
-                return 1;
-        default:
-                return 0;
-        }
-}
-
-static int
-sun_disk_get_flag (const PedDisk *disk, PedDiskFlag flag)
-{
-        SunDiskData *disk_specific = disk->disk_specific;
-        switch (flag) {
-        case PED_DISK_CYLINDER_ALIGNMENT:
-                return disk_specific->cylinder_alignment;
-        default:
-                return 0;
-        }
-}
-
-static int
-sun_disk_is_flag_available (const PedDisk *disk, PedDiskFlag flag)
-{
-        switch (flag) {
-        case PED_DISK_CYLINDER_ALIGNMENT:
-               return 1;
-        default:
-               return 0;
-        }
-}
-
 static int
 _check_geometry_sanity (PedDisk* disk, SunRawLabel* label)
 {
@@ -355,11 +313,10 @@ sun_read (PedDisk* disk)
 	int i;
 	PedPartition* part;
 	PedSector end, start, block;
-	PedConstraint* constraint_exact;
 
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (disk->dev != NULL, return 0);
-	PED_ASSERT (disk->disk_specific != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (disk->dev != NULL);
+	PED_ASSERT (disk->disk_specific != NULL);
 
 	disk_data = (SunDiskData*) disk->disk_specific;
 
@@ -408,10 +365,14 @@ sun_read (PedDisk* disk)
 		part->num = i + 1;
 		part->fs_type = ped_file_system_probe (&part->geom);
 
-		constraint_exact = ped_constraint_exact (&part->geom);
-		if (!ped_disk_add_partition (disk, part, constraint_exact))
+		PedConstraint *constraint_exact
+			= ped_constraint_exact (&part->geom);
+		if (constraint_exact == NULL)
 			goto error;
+		bool ok = ped_disk_add_partition (disk, part, constraint_exact);
 		ped_constraint_destroy (constraint_exact);
+		if (!ok)
+			goto error;
 	}
 
 	return 1;
@@ -446,8 +407,8 @@ sun_write (const PedDisk* disk)
 	PedPartition*		part;
 	int			i;
 
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (disk->dev != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (disk->dev != NULL);
 
 	void *s0;
 	if (!ptt_read_sector (disk->dev, 0, &s0))
@@ -560,7 +521,6 @@ sun_partition_new (const PedDisk* disk, PedPartitionType part_type,
 
 	return part;
 
-	free (sun_data);
 error_free_part:
 	free (part);
 error:
@@ -594,7 +554,7 @@ sun_partition_duplicate (const PedPartition* part)
 static void
 sun_partition_destroy (PedPartition* part)
 {
-	PED_ASSERT (part != NULL, return);
+	PED_ASSERT (part != NULL);
 
 	if (ped_partition_is_active (part))
 		free (part->disk_specific);
@@ -641,9 +601,9 @@ sun_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 {
 	SunPartitionData*		sun_data;
 
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk_specific != NULL, return 0);
-	PED_ASSERT (ped_partition_is_flag_available (part, flag), return 0);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk_specific != NULL);
+	PED_ASSERT (ped_partition_is_flag_available (part, flag));
 
 	sun_data = part->disk_specific;
 
@@ -690,13 +650,13 @@ sun_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 }
 
 
-static int
+static int _GL_ATTRIBUTE_PURE
 sun_partition_get_flag (const PedPartition* part, PedPartitionFlag flag)
 {
 	SunPartitionData*       sun_data;
 
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk_specific != NULL, return 0);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk_specific != NULL);
 
 	sun_data = part->disk_specific;
 
@@ -804,12 +764,9 @@ _get_lax_constraint (PedDisk* disk)
 static int
 sun_partition_align (PedPartition* part, const PedConstraint* constraint)
 {
-        PED_ASSERT (part != NULL, return 0);
+        PED_ASSERT (part != NULL);
 
-        SunDiskData *disk_specific = part->disk->disk_specific;
-
-        if (disk_specific->cylinder_alignment &&
-            _ped_partition_attempt_align (part, constraint,
+        if (_ped_partition_attempt_align (part, constraint,
                                           _get_strict_constraint (part->disk)))
                 return 1;
         if (_ped_partition_attempt_align (part, constraint,
@@ -879,9 +836,9 @@ sun_alloc_metadata (PedDisk* disk)
 	SunDiskData*	disk_data;
 	PedConstraint*	constraint_any;
 
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (disk->disk_specific != NULL, return 0);
-	PED_ASSERT (disk->dev != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (disk->disk_specific != NULL);
+	PED_ASSERT (disk->dev != NULL);
 
 	constraint_any = ped_constraint_any (disk->dev);
 
@@ -920,13 +877,9 @@ error:
 PT_define_limit_functions (sun)
 
 static PedDiskOps sun_disk_ops = {
-	clobber:		NULL_IF_DISCOVER_ONLY (sun_clobber),
+	clobber:		NULL,
 	write:			NULL_IF_DISCOVER_ONLY (sun_write),
 
-	disk_set_flag:          sun_disk_set_flag,
-	disk_get_flag:          sun_disk_get_flag,
-	disk_is_flag_available: sun_disk_is_flag_available,
-
 	get_partition_alignment: sun_get_partition_alignment,
 
 	partition_set_name:		NULL,
@@ -945,7 +898,7 @@ static PedDiskType sun_disk_type = {
 void
 ped_disk_sun_init ()
 {
-	PED_ASSERT (sizeof (SunRawLabel) == 512, return);
+	PED_ASSERT (sizeof (SunRawLabel) == 512);
 	ped_disk_type_register (&sun_disk_type);
 }
 
