@@ -1,6 +1,6 @@
 /*
     parted - a frontend to libparted
-    Copyright (C) 1999-2001, 2007, 2009 Free Software Foundation, Inc.
+    Copyright (C) 1999-2001, 2007, 2009-2014 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,21 +37,13 @@
 #include <wchar.h>
 #include <wctype.h>
 
-#else /* ENABLE_NLS */
-
-#ifdef wchar_t
-#undef wchar_t
-#endif
-
-#define wchar_t char
-
 #endif /* !ENABLE_NLS */
 
 #include "strlist.h"
 
 #define MIN(a,b)	( (a<b)?  a : b )
 
-int
+static int _GL_ATTRIBUTE_PURE
 wchar_strlen (const wchar_t* str)
 {
 #ifdef ENABLE_NLS
@@ -61,7 +53,7 @@ wchar_strlen (const wchar_t* str)
 #endif
 }
 
-wchar_t*
+static wchar_t * _GL_ATTRIBUTE_PURE
 wchar_strchr (const wchar_t* str, char ch)
 {
 #ifdef ENABLE_NLS
@@ -71,7 +63,7 @@ wchar_strchr (const wchar_t* str, char ch)
 #endif
 }
 
-int
+static int _GL_ATTRIBUTE_PURE
 wchar_strcasecmp (const wchar_t* a, const wchar_t* b)
 {
 #ifdef ENABLE_NLS
@@ -81,7 +73,7 @@ wchar_strcasecmp (const wchar_t* a, const wchar_t* b)
 #endif
 }
 
-int
+static int _GL_ATTRIBUTE_PURE
 wchar_strncasecmp (const wchar_t* a, const wchar_t* b, size_t n)
 {
 #ifdef ENABLE_NLS
@@ -91,7 +83,7 @@ wchar_strncasecmp (const wchar_t* a, const wchar_t* b, size_t n)
 #endif
 }
 
-wchar_t*
+static wchar_t * _GL_ATTRIBUTE_PURE
 wchar_strdup (const wchar_t* str)
 {
 #ifdef ENABLE_NLS
@@ -120,14 +112,14 @@ gettext_to_wchar (const char* str)
 
 	memset(&ps, 0, sizeof (ps));
 	status = mbsrtowcs(result, &str, count, &ps);
-	if (status == (size_t) -1)
+	if (str != NULL)
 		goto error;
 
-	result = xrealloc (result, (wcslen (result) + 1) * sizeof (wchar_t));
+	result = xrealloc (result, (status + 1) * sizeof (wchar_t));
 	return result;
 
 error:
-	printf ("Error during translation: %s\n", strerror (errno));
+	fprintf (stderr, "Error during translation: %s\n", strerror (errno));
 	exit (EXIT_FAILURE);
 }
 
@@ -176,7 +168,7 @@ wchar_to_str (const wchar_t* str, size_t count)
 	return result;
 
 error:
-	printf ("Error during translation: %s\n", strerror (errno));
+	fprintf (stderr, "Error during translation: %s\n", strerror (errno));
 	exit (EXIT_FAILURE);
 }
 
@@ -196,10 +188,10 @@ wchar_to_str (const wchar_t* str, size_t count)
 #endif /* !ENABLE_NLS */
 
 static void
-print_wchar (const wchar_t* str, size_t count)
+print_wchar (const wchar_t* str, size_t count, FILE *fp)
 {
 	char*	tmp = wchar_to_str (str, count);
-	printf ("%s", tmp);
+	fprintf (fp, "%s", tmp);
 	free (tmp);
 }
 
@@ -381,13 +373,13 @@ str_list_convert (const StrList* list)
 }
 
 void
-str_list_print (const StrList* list)
+str_list_print (const StrList* list, FILE *fp)
 {
 	const StrList*	walk;
 
 	for (walk=list; walk; walk=walk->next) {
 		if (walk->str)
-			print_wchar (walk->str, 0);
+                        print_wchar (walk->str, 0, fp);
 	}
 }
 
@@ -438,7 +430,7 @@ is_space (wchar_t c)
 
 void
 str_list_print_wrap (const StrList* list, int line_length, int offset,
-		     int indent)
+		     int indent, FILE *fp)
 {
 	const StrList*	walk;
 	const wchar_t*	str;
@@ -449,7 +441,7 @@ str_list_print_wrap (const StrList* list, int line_length, int offset,
 	int		search_result;
 	int		line_break;
 
-	PED_ASSERT (line_length - indent > 10, return);
+	PED_ASSERT (line_length - indent > 10);
 
 	line_left = line_length - offset;
 
@@ -489,19 +481,19 @@ str_list_print_wrap (const StrList* list, int line_length, int offset,
 			     cut_right++);
 
 			if (cut_left > 0)
-				print_wchar (str, cut_left + 1);
+				print_wchar (str, cut_left + 1, fp);
 
 			str += cut_right;
 			str_len -= cut_right;
 			line_left = line_length - indent;
 
 			if (walk->next || *str)
-				printf ("\n%*s", indent, "");
+				fprintf (fp, "\n%*s", indent, "");
 			else if (line_break)
-				putchar ('\n');
+				fputc ('\n', fp);
 		}
 
-		print_wchar (str, 0);
+		print_wchar (str, 0, fp);
 		line_left -= wchar_strlen (str);
 	}
 }
